///////////////////////////////////////////////////////////////////////////
// Filename : MP3_Decoder_tb.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  : 
//
//
#include "MP3_Decoder.h"
#include "MP3_Decoder_tb.h"

#ifdef ITHREAD
extern "C" {
    void demand_on();
}
#endif

#ifdef _DEBUG_VCD
#include "monitor_mp3.h"
#endif

#include "SMC.h"

// UUT Instantiate ////////////////////////////
mp3_decoder u_mp3_decoder("u_mp3_decoder");

int sc_main(int argc, char* argv[])
{
    const char *inputFileName;

    if( argc != 2 ){
        cout << "Usage: MP3_Decoder <filename.mp3>" << endl;
        exit(-1);
        }
    else
        inputFileName = argv[1];
        
    sc_clock                CLK("clk", CLK_PERIOD);     // Clock
    sc_signal<sc_bit>       rst;            // Reset
    sc_signal<sc_bit>       stop;

    sc_signal<sc_bit>       mp3_ack;
    sc_signal<sc_bit>       mp3_req;
    sc_signal<sc_bv<8> >    mp3_data;
    sc_signal<int>          mp3_count;

#ifdef _DEBUG_OUT
    system("del dbg_*.txt");
#endif

    // UUT : MP3_Decoder ////////////////////////////////////////////////
    //mp3_decoder   u_mp3_decoder("u_mp3_decoder");
    u_mp3_decoder.clk(CLK);
    u_mp3_decoder.rst(rst);
    u_mp3_decoder.stop(stop);
    u_mp3_decoder.mp3_req(mp3_req);
    u_mp3_decoder.mp3_ack(mp3_ack);
    u_mp3_decoder.mp3_data(mp3_data);

    // SMC //////////////////////////////////////////////////////////////
    smc u_smc("u_smc");
    u_smc.Open( inputFileName ); // Open input stream.

    u_smc.clk(CLK);             // Clock
    u_smc.rst(rst);             // Reset
    u_smc.eof(stop);            // EOF of Input stream
    u_smc.mp3_ack(mp3_ack);
    u_smc.mp3_req(mp3_req);
    u_smc.mp3_data(mp3_data);
    u_smc.mp3_count(mp3_count);


    // MON : MP3 ////////////////////////////////////////////////////////
#ifdef _DEBUG_VCD
    monitor_mp3 u_monitor_mp3("u_monitor_mp3");
    u_monitor_mp3.clk(CLK);
    u_monitor_mp3.rst(rst);
    u_monitor_mp3.stop(stop);
    u_monitor_mp3.mp3_req(mp3_req);
    u_monitor_mp3.mp3_ack(mp3_ack);
    u_monitor_mp3.mp3_data(mp3_data);
    u_monitor_mp3.mp3_count(mp3_count);
#endif

    // Simulator Command ///////////////////////////////
    rst = (sc_bit)'0';
    sc_start(CLK_PERIOD * 2);

    rst = (sc_bit)'1';
#ifdef ITHREAD
    demand_on();
#endif
    sc_start(-1);

#ifdef _DEBUG_VCD
    sc_close_vcd_trace_file(u_monitor_mp3.tf);
    sc_close_vcd_trace_file(u_mp3_decoder.u_monitor_decoder_ctl->tf);
    sc_close_vcd_trace_file(u_mp3_decoder.u_monitor_header->tf);
    sc_close_vcd_trace_file(u_mp3_decoder.u_monitor_scale->tf);
    sc_close_vcd_trace_file(u_mp3_decoder.u_frame_buffer->tf);

    system("winwave decoder_ctl.vcd");
#endif

    return(0);
}
