///////////////////////////////////////////////////////////////////////////
// Filename : SMC.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#ifndef SOFT
    extern "C" {
        #include "isave/iswbuf.h"
        #include "mp3psg.h"
        int psg_read();
        int psg_write();
        int __lprintf();
        int __dprintf();
    }
#endif

#ifdef ITHREAD
    extern isave_swbuf_t i2si_buf;
#endif

#include "MP3_Decoder.h"
#include "SMC.h"

// Fill Frame Buffer /////////////////////////////////////////////////////
void smc::proc_smc()
{
#ifndef SOFT
    int status;
    static int rdstatus=0;
    static UINT data[512];
    static int dptr=0;
    static int remain=0;
    static union {
        unsigned char b[4];
        unsigned long d;
    } split;
#endif

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
#ifndef SOFT
        __lprintf("v0.4\n");
#endif
        count = 0;
        eof = (bool)false;

        mp3_ack = (sc_bit)'0';
        SMC_State = S_SMC_WAIT;
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(SMC_State)
        {
            case S_SMC_WAIT:
                if (mp3_req.read()==(sc_bit)'1')
                {
                    // END-OF-INPUT STREAM : Wait forever !!!!!
#ifdef SOFT
                    if (feof(mp3_fp)!=0)
                    {
                        eof = (bool)true;
                    }
                    else
                    {
                        mp3_data  = fgetc(mp3_fp);
                        eof = (bool)false;
                    }
#else
                    if(remain>0)
                    {
                        remain--;
                        mp3_data = split.b[remain];
                    }
#ifndef ITHREAD
                    else if(rdstatus==0)
                    {
                        do {
                            rdstatus = psg_read(IFLIB_AD_RST(0));
                            if(rdstatus < 300)
                            {
                                psg_write(0x800,1);
                            }
                        } while(rdstatus<300);
                        psg_write(0x800,0);
                        rdstatus=psg_read(IFLIB_AD_RST(0));
                        while(rdstatus>=512) __lprintf("fifo full\n");
                        iflib_i2s_IMV2_dd(0,0,data,rdstatus,&status,1);
                        __dprintf("read %d word\n",rdstatus);
                        {
                            for(int i=0; i<rdstatus; i++)
                            {
                                __dprintf("%x ",data[i]);
                            }
                            __dprintf("\n");
                        }
                        remain=3;
                        dptr=0;
                        rdstatus--;
                        split.d=(int)data[dptr];
                        mp3_data=split.b[remain];
                    }
#endif
                    else
                    {
#ifdef ITHREAD
                        while( iswbuf_get(&i2si_buf,
                            (unsigned int*)&(split.d)) < 0 );
#endif
                        remain=3;
#ifdef ITHREAD
                        dptr++;
                        rdstatus--;
                        split.d=(int)data[dptr];
#endif
                        mp3_data=split.b[remain];
                    }
#endif

                    SMC_State = S_SMC_REQ;
                }
                break;

            case S_SMC_REQ:
                mp3_ack = (sc_bit)'1';
                SMC_State = S_SMC_ACK;
                break;

            case S_SMC_ACK:
                if (mp3_req.read()==(sc_bit)'0')
                {
                    count++;
                    mp3_count = count;
                    mp3_ack = (sc_bit)'0';
                    SMC_State = S_SMC_WAIT;
                }
                break;
        }
    }
}
