///////////////////////////////////////////////////////////////////////////
// Filename : monitor_header.h
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : Jun.,2000
// Comment  : Waveform Monitor
//
//
#ifndef MONITOR_HEADER_INC
#define MONITOR_HEADER_INC

#include "systemc.h"

SC_MODULE(monitor_header)
{
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Header Start
    sc_in<sc_bit>       ready;  // Header Ready

    sc_in<sc_bit>       ch;     // Channel
    sc_in<sc_bit>       gr;     // Granule

    sc_in<sc_bv<9> >    big_values;
    sc_in<sc_bv<2> >    block_type;
    sc_in<sc_bit>       count1table_select;
    sc_in<sc_bv<8> >    global_gain;
    sc_in<sc_bv<3> >    subblock_gain0;
    sc_in<sc_bv<3> >    subblock_gain1;
    sc_in<sc_bv<3> >    subblock_gain2;
    sc_in<sc_bit>       mixed_block_flag;
    sc_in<sc_bv<12> >   part2_3_length;
    sc_in<sc_bit>       preflag;
    sc_in<sc_bv<4>  >   region0_count;
    sc_in<sc_bv<3>  >   region1_count;
    sc_in<sc_bv<9> >    scalefac_compress;
    sc_in<sc_bit>       scalefac_scale;
    sc_in<sc_bit>       scfsi0;
    sc_in<sc_bit>       scfsi1;
    sc_in<sc_bit>       scfsi2;
    sc_in<sc_bit>       scfsi3;
    sc_in<sc_bv<5> >    table_select0;
    sc_in<sc_bv<5> >    table_select1;
    sc_in<sc_bv<5> >    table_select2;
    sc_in<sc_bit>       window_switching_flag;
    sc_in<sc_bit>       stereo;
    sc_in<sc_bv<4> >    bitrate_index;
    sc_in<sc_bv<2> >    mode;
    sc_in<sc_bv<2> >    mode_ext;
    sc_in<sc_bit>       version;
    sc_in<sc_bv<2> >    sampling_frequency;

    // Process ///////////////////////////////////////////////////////////
    void proc_monitor();
    void print_header();

    // Signals to be taced ///////////////////////////////////////////////
//  bool        t_clk;
//  bool        t_rst;
    bool        t_start;
    bool        t_ready;
    bool        t_ch;
    bool        t_gr;
    sc_uint<9>  t_big_values;
    sc_uint<2>  t_block_type;
    bool        t_count1table_select;
    sc_uint<8>  t_global_gain;
    sc_uint<3>  t_subblock_gain0;
    sc_uint<3>  t_subblock_gain1;
    sc_uint<3>  t_subblock_gain2;
    bool        t_mixed_block_flag;
    sc_uint<12> t_part2_3_length;
    bool        t_preflag;
    sc_uint<4>  t_region0_count;
    sc_uint<3>  t_region1_count;
    sc_uint<9>  t_scalefac_compress;
    bool        t_scalefac_scale;
    bool        t_scfsi0;
    bool        t_scfsi1;
    bool        t_scfsi2;
    bool        t_scfsi3;
    sc_uint<5>  t_table_select0;
    sc_uint<5>  t_table_select1;
    sc_uint<5>  t_table_select2;
    bool        t_window_switching_flag;
    bool        t_stereo;
    sc_uint<4>  t_bitrate_index;
    sc_uint<2>  t_mode;
    sc_uint<2>  t_mode_ext;
    bool        t_version;
    sc_uint<2>  t_sampling_frequency;

    // Waveform //////////////////////////////////////////////////////////////
    public:
        sc_trace_file*  tf;

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(monitor_header)
    {
        SC_METHOD(print_header);
        sensitive << ready;

        SC_METHOD(proc_monitor);
        sensitive
            << clk
            << rst
            << start
            << ready
            << ch
            << gr
            << big_values
            << block_type
            << count1table_select
            << global_gain
            << subblock_gain0
            << subblock_gain1
            << subblock_gain2
            << mixed_block_flag
            << part2_3_length
            << preflag
            << region0_count
            << region1_count
            << scalefac_compress
            << scalefac_scale
            << scfsi0
            << scfsi1
            << scfsi2
            << scfsi3
            << table_select0
            << table_select1
            << table_select2
            << window_switching_flag
            << stereo
            << mode
            << mode_ext
            << version
            << sampling_frequency;

        // Waaveform ///////////////////////////////
        tf = sc_create_vcd_trace_file("header");
        ((vcd_trace_file*)tf)->sc_set_vcd_time_unit(-9);

//      sc_trace(tf, t_clk, "clk");
//      sc_trace(tf, t_rst, "rst");
        sc_trace(tf, t_start, "start");
        sc_trace(tf, t_ready, "ready");
        sc_trace(tf, t_ch, "channel");
        sc_trace(tf, t_gr, "granule");
        sc_trace(tf, t_big_values, "big_values");
        sc_trace(tf, t_block_type, "block_type");
        sc_trace(tf, t_count1table_select, "count1table_select");
        sc_trace(tf, t_global_gain, "global_gain");
        sc_trace(tf, t_subblock_gain0, "subblock_gain0");
        sc_trace(tf, t_subblock_gain1, "subblock_gain1");
        sc_trace(tf, t_subblock_gain2, "subblock_gain2");
        sc_trace(tf, t_mixed_block_flag, "mixed_block_flag");
        sc_trace(tf, t_part2_3_length, "part2_3_length");
        sc_trace(tf, t_preflag, "preflag");
        sc_trace(tf, t_region0_count, "region0_count");
        sc_trace(tf, t_region1_count, "region1_count");
        sc_trace(tf, t_scalefac_compress, "scalefac_compress");
        sc_trace(tf, t_scalefac_scale, "scalefac_scale");
        sc_trace(tf, t_scfsi0, "scfsi0");
        sc_trace(tf, t_scfsi1, "scfsi1");
        sc_trace(tf, t_scfsi2, "scfsi2");
        sc_trace(tf, t_scfsi3, "scfsi3");
        sc_trace(tf, t_table_select0, "table_select0");
        sc_trace(tf, t_table_select1, "table_select1");
        sc_trace(tf, t_table_select2, "table_select2");
        sc_trace(tf, t_window_switching_flag, "window_switching_flag");
        sc_trace(tf, t_stereo, "stereo");
        sc_trace(tf, t_bitrate_index, "bitrate_index");
        sc_trace(tf, t_mode, "mode");
        sc_trace(tf, t_mode_ext, "mode_ext");
        sc_trace(tf, t_version, "version");
        sc_trace(tf, t_sampling_frequency, "sampling_frequency");
    }
};

#endif
