///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Decoder_Mem.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#include "MP3_Decoder.h"
#include "sc_III_Decoder_Mem.h"

// Initialize Frame Buffer /////////////////////////////////////////////////////
void decoder_mem::proc_mem_init()
{
    int ch, sb, ss, cb, win;

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        m_init_done = (sc_bit)'0';
        Mem_Init_State = S_MEM_INIT_WAIT;
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Mem_Init_State.read())
        {
            case S_MEM_INIT_WAIT:
                if (mem_init.read()==(sc_bit)'1')
                {
                    m_init_done = (sc_bit)'0';
                    Mem_Init_State = S_MEM_INIT_BUSY;
                }
                break;

            case S_MEM_INIT_BUSY:
                // Initialize Decoder Memory
                for (ch=0; ch<2; ch++)
                {
                    for (cb=0; cb<23; cb++)
                        scalefac[ch].l[cb] = 0;
                    for (cb=0; cb<13; cb++)
                        for (win=0; win<3; win++)
                            scalefac[ch].s[win][cb] = 0;

                    for (sb=0; sb < SBLIMIT; sb++)
                        for (ss=0; ss < SSLIMIT; ss++)
                        {
                            pcm_sample[ch][ss][sb] = 0;
                            ram[ch][sb][ss] = 0.0;
                            prevblck[ch][sb][ss] = 0.0;
                        }
                }

                for (ss=0; ss < SSLIMIT*2; ss++)
                    rawout[ss] = 0.0;

                m_init_done = (sc_bit)'1';
                Mem_Init_State = S_MEM_INIT_DONE;
                break;

            case S_MEM_INIT_DONE:
                if (mem_init.read()==(sc_bit)'0')
                {
                    m_init_done = (sc_bit)'0';
                    Mem_Init_State = S_MEM_INIT_WAIT;
                }
                break;
        }
    }
}

// Read/Write Scale factor data
long int decoder_mem::get_scalefac(int ch, int window, int cb, int block_type)
{
    if (block_type==0)
        return(scalefac[ch].l[cb]);
    else
        return(scalefac[ch].s[window][cb]);
};

void decoder_mem::put_scalefac(long int data, int ch, int window, int cb, int block_type)
{
    if (block_type==0)
        scalefac[ch].l[cb] = data;
    else
        scalefac[ch].s[window][cb] = data;
};

// Read/Write Huffman decoded
long decoder_mem::get_is(int ch, int sb, int ss)
{
    return((long)ram[ch][sb][ss]);
};

void decoder_mem::put_is(long data, int ch, int sb, int ss)
{
    ram[ch][sb][ss] = (double)data;
};

// Read/Write Requantized data
double decoder_mem::get_ro(int ch, int sb, int ss)
{
    return(ram[ch][sb][ss]);
};

void decoder_mem::put_ro(double data, int ch, int sb, int ss)
{
    ram[ch][sb][ss] = data;
};

// Read/Write Stero processed data
double decoder_mem::get_lr(int ch, int sb, int ss)
{
    return(ram[ch][sb][ss]);
}

void decoder_mem::put_lr(double data, int ch, int sb, int ss)
{
    ram[ch][sb][ss] = data;
};

// Read/Write Reordered data
double decoder_mem::get_re(int ch, int sb, int ss)
{
    return(ram[ch][sb][ss]);
};

void decoder_mem::put_re(double data, int ch, int sb, int ss)
{
    ram[ch][sb][ss] = data;
};

// Read/Write Antialiased data
double decoder_mem::get_hybridIn(int ch, int sb, int ss)
{
    return(ram[ch][sb][ss]);
};

void decoder_mem::put_hybridIn(double data, int ch, int sb, int ss)
{
    ram[ch][sb][ss] = data;
};

// Read/Write Hybrid-Prev
double decoder_mem::get_prevblck(int ch, int sb, int ss)
{
    return(prevblck[ch][sb][ss]);
};

void decoder_mem::put_prevblck(double data, int ch, int sb, int ss)
{
    prevblck[ch][sb][ss] = data;
};

// Read/Write Hybrid-Rawout
double decoder_mem::get_rawout(int ss)
{
    return(rawout[ss]);
};

void decoder_mem::put_rawout(double data, int ss)
{
    rawout[ss] = data;
};

// Read/Write Hybrid
double decoder_mem::get_hybridOut(int ch, int sb, int ss)
{
    return(ram[ch][sb][ss]);
};

void decoder_mem::put_hybridOut(double data, int ch, int sb, int ss)
{
    ram[ch][sb][ss] = data;
};

// Read/Write PCM Sample
short decoder_mem::get_pcm_sample(int ch, int sb, int ss)
{
    return(pcm_sample[ch][ss][sb]);
};

void decoder_mem::put_pcm_sample(short data, int ch, int sb, int ss)
{
    pcm_sample[ch][ss][sb] = data;
};
