///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Frame_buffer.h
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  : Frame Buffer
//
//

#ifndef FRAME_BUFFER_INC
#define FRAME_BUFFER_INC

#include "systemc.h"
#include "sc_III_Common.h"

enum S_Fill_Buff_Type {
    S_FILL_BUFF_WAIT,
    S_FILL_BUFF_REQ,
    S_FILL_BUFF_ACK
};

SC_MODULE(frame_buffer)
{
    sc_in_clk           clk;                // Clock
    sc_in<sc_bit>       rst;                // Reset

    sc_in<sc_bit>       fill_stream;
    sc_out<sc_bit>      stream_full;

    sc_in<sc_bit>       mp3_ack;
    sc_out<sc_bit>      mp3_req;
    sc_in<sc_bv<8> >    mp3_data;

    // Process ///////////////////////////////////////////////////////////
    void proc_fill_buffer();

    // Signals ///////////////////////////////////////////////////////////
    sc_signal<S_Fill_Buff_Type> Fill_Buff_State;

    // MP3 Stream Data : Frame Memory ////////////////////////////////////
    int         nBegin_FIFO, nEnd_FIFO;
    int         frm_bit_index;
    long        frm_totbit;
    sc_bv<8>    frm_mem[1024];

    // MP3 Main Data Buffer //////////////////////////////////////////////
    int         buf_bit_index;
    long        buf_totbit, buf_offset, buf_byte_index;
    sc_bv<8>        buf_mem[4096];

    // Member functions //////////////////////////////////////////////////
    public:
    unsigned long get1bit();
    unsigned long getbits(int N);
    unsigned long sstell();
    unsigned long end_bs();

    void hputbuf(unsigned int val);
    unsigned long hget1bit();
    unsigned long hgetbits(int N);
    unsigned long hsstell();
    void rewindNbytes(int N);
    void rewindNbits(int N);

#ifdef _DEBUG_VCD
    // Waveform //////////////////////////////////////////////////////////////
    public:
        sc_trace_file*  tf;
#endif

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(frame_buffer)
    {
        SC_METHOD(proc_fill_buffer);
        sensitive << clk << rst;

#ifdef _DEBUG_VCD
        // Waaveform ///////////////////////////////
        tf = sc_create_vcd_trace_file("frame_buffer");
        ((vcd_trace_file*)tf)->sc_set_vcd_time_unit(-9);

        sc_trace(tf, nBegin_FIFO,   "nBegin_FIFO");
        sc_trace(tf, nEnd_FIFO,     "nEnd_FIFO");
        sc_trace(tf, buf_offset,    "buf_offset");
        sc_trace(tf, buf_totbit,    "buf_totbit");
#endif
    }
};

#endif
