///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_HeaderInfo.h
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//
#ifndef III_HEADER_INC
#define III_HEADER_INC

#include "systemc.h"
#include "sc_III_Common.h"

enum tHeader_State {
    S_HEADER_WAIT,
    S_HEADER_DECODE,
    S_HEADER_DOWNLOAD,
    S_HEADER_BUSY };

SC_MODULE(iii_headerinfo)
{
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Header Start
    sc_out<sc_bit>      ready;  // Header Ready

    sc_in<sc_bit>       ch;     // Channel
    sc_in<sc_bit>       gr;     // Granule

    sc_out<sc_bv<4> >   bitrate_index;
    sc_out<sc_bv<9> >   big_values;
    sc_out<sc_bv<2> >   block_type;
    sc_out<sc_bit>      count1table_select;
    sc_out<sc_bv<8> >   global_gain;
    sc_out<sc_bv<3> >   subblock_gain0;
    sc_out<sc_bv<3> >   subblock_gain1;
    sc_out<sc_bv<3> >   subblock_gain2;
    sc_out<sc_bit>      mixed_block_flag;
    sc_out<sc_bv<12> >  part2_3_length;
    sc_out<sc_bit>      preflag;
    sc_out<sc_bv<4> >   region0_count;
    sc_out<sc_bv<3> >   region1_count;
    sc_out<sc_bv<9> >   scalefac_compress;
    sc_out<sc_bit>      scalefac_scale;
    sc_out<sc_bit>      scfsi0;
    sc_out<sc_bit>      scfsi1;
    sc_out<sc_bit>      scfsi2;
    sc_out<sc_bit>      scfsi3;
    sc_out<sc_bv<5> >   table_select0;
    sc_out<sc_bv<5> >   table_select1;
    sc_out<sc_bv<5> >   table_select2;
    sc_out<sc_bit>      window_switching_flag;
    sc_out<sc_bit>      stereo;
    sc_out<sc_bv<2> >   mode;
    sc_out<sc_bv<2> >   mode_ext;
    sc_out<sc_bit>      version;
    sc_out<sc_bv<2> >   sampling_frequency;

    // Process ///////////////////////////////////////////////////////////
    void proc_header();
    void proc_header_out();

    // Signals ///////////////////////////////////////////////////////////
    sc_signal<tHeader_State>    Header_State;   // FSM State Variable

    // Globals ///////////////////////////////////////////////////////////
    frame_params        fr_ps;          /* Frame Parameters */
    layer               info;           /* Frame's Header Information */
    III_side_info_t     III_side_info;  /* Layer III Side info. of each frame */

    unsigned int        old_crc;
    int                 nSlots;
    int                 main_data_end;
    int                 flush_main;
    int                 bytes_to_discard ;
    int                 frame_start;

    // Member functions //////////////////////////////////////////////////
    unsigned long seek_sync();
    void decode_info(frame_params *fr_ps);
    void hdr_to_frps(frame_params *fr_ps);
    void buffer_CRC(unsigned int  *old_crc);
    void III_get_side_info(III_side_info_t *si, frame_params *fr_ps );
    int  main_data_slots(frame_params fr_ps);

    // Constructor ///////////////////////////////////////////////////////
    SC_CTOR(iii_headerinfo)
    {
        SC_METHOD(proc_header);
        sensitive << clk << rst;

        SC_METHOD(proc_header_out);
        sensitive << ch << gr;

        // Initializer
        {
            for (int ich=0; ich<2; ich++)
                for (int igr=0; igr<2; igr++)
                {
                    III_side_info.ch[ich].gr[igr].big_values = 0;
                    III_side_info.ch[ich].gr[igr].block_type = 0;
                    III_side_info.ch[ich].gr[igr].count1table_select = 0;
                    III_side_info.ch[ich].gr[igr].global_gain = 0;
                    III_side_info.ch[ich].gr[igr].subblock_gain[0] = 0;
                    III_side_info.ch[ich].gr[igr].subblock_gain[1] = 0;
                    III_side_info.ch[ich].gr[igr].subblock_gain[2] = 0;
                    III_side_info.ch[ich].gr[igr].mixed_block_flag = 0;
                    III_side_info.ch[ich].gr[igr].part2_3_length = 0;
                    III_side_info.ch[ich].gr[igr].preflag = 0;
                    III_side_info.ch[ich].gr[igr].region0_count = 0;
                    III_side_info.ch[ich].gr[igr].region1_count = 0;
                    III_side_info.ch[ich].gr[igr].scalefac_compress = 0;
                    III_side_info.ch[ich].gr[igr].scalefac_scale = 0;
                    III_side_info.ch[ich].scfsi[0] = 0;
                    III_side_info.ch[ich].scfsi[1] = 0;
                    III_side_info.ch[ich].scfsi[2] = 0;
                    III_side_info.ch[ich].scfsi[3] = 0;
                    III_side_info.ch[ich].gr[igr].table_select[0] = 0;
                    III_side_info.ch[ich].gr[igr].table_select[1] = 0;
                    III_side_info.ch[ich].gr[igr].table_select[2] = 0;
                    III_side_info.ch[ich].gr[igr].window_switching_flag = 0;
                }
        
            fr_ps.header = (layer *)&info;
            fr_ps.stereo = 0;
            fr_ps.header->mode = 0;
            fr_ps.header->mode_ext = 0;
            fr_ps.header->version = 0;
            fr_ps.header->sampling_frequency = 0;
        }
    }
};

#endif
