///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Huffman.h
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//
#ifndef III_HUFFMAN_INC
#define III_HUFFMAN_INC

#include "systemc.h"
#include "sc_III_Common.h"

enum tHuffman_State {
    S_HUFFMAN_WAIT,
    S_HUFFMAN_DECODE,
    S_HUFFMAN_DOWNLOAD,
    S_HUFFMAN_BUSY };

// Definition for Huffman Decoder ///////////////////////////////////
#define HUFFBITS    unsigned long int
#define HTN         34
#define MXOFF       250
 
struct huffcodetab {
    char tablename[3];      // string, containing table_description
    unsigned int xlen;      // max. x-index+
    unsigned int ylen;      // max. y-index+
    unsigned int linbits;   // number of linbits
    unsigned int linmax;    // max number to be stored in linbits
    int ref;                // a positive value indicates a reference
    HUFFBITS *table;        // pointer to array[xlen][ylen]
    unsigned char *hlen;    // pointer to array[xlen][ylen]
    unsigned char(*val)[2]; // decoder tree
    unsigned int treelen;   // length of decoder tree
};

SC_MODULE(iii_huffman)
{
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Huffman Start
    sc_out<sc_bit>      ready;  // Huffman Ready

    sc_in<sc_bit>       ch;     // Channel
    sc_in<sc_bit>       gr;     // Granule

    sc_in<sc_bv<9> >    big_values;
    sc_in<sc_bit>       count1table_select;
    sc_in<sc_bv<12> >   part2_3_length;
    sc_in<sc_bv<4>  >   region0_count;
    sc_in<sc_bv<3>  >   region1_count;
    sc_in<sc_bv<2> >    sampling_frequency;
    sc_in<sc_bv<5> >    table_select0;
    sc_in<sc_bv<5> >    table_select1;
    sc_in<sc_bv<5> >    table_select2;
    sc_in<sc_bit>       version;
    sc_in<unsigned long>    part2_start;
    sc_in<sc_bv<2> >    block_type;
    sc_in<sc_bit>       window_switching_flag;
    
    // Process ///////////////////////////////////////////////////////////
    void proc_huffman();

    // Signals ///////////////////////////////////////////////////////////
    sc_signal<tHuffman_State>   Huffman_State;  // FSM State Variable

    // Member functions //////////////////////////////////////////////////
    void III_hufman_decode(
            int         version,
            int         big_values,
            int         table_select0,
            int         table_select1,
            int         table_select2,
            int         count1table_select,
            int         sampling_frequency,
            int         region0_count,
            int         region1_count,
            int         part2_3_length,
            int         window_switching_flag,
            int         block_type,
            int         ch,
            int         gr,
            int         part2_start);

    int huffman_decoder(
            struct huffcodetab  *h,
            int                 *x,
            int                 *y,
            int                 *v,
            int                 *w);

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(iii_huffman)
    {
        SC_METHOD(proc_huffman);
        sensitive << clk << rst;
    }
};

#endif
