///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_OutFifo.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#ifndef SOFT
    extern "C" {
        #include "isave/iswbuf.h"
        #include "isave/ihwbuf.h"
        #include "ilibdd.h"
        #include "iflib_i2s_OMV2_dd.h"
        int psg_read();
        int __lprintf();
        int __dprintf();
    }
#endif

#ifdef OTHREAD
    extern isave_swbuf_t i2so_buf;
#endif

#include "MP3_Decoder.h"
#include "sc_III_OutFifo.h"
#include "WavImage.h"

#include "sc_III_Decoder_Mem.h"

void iii_outfifo::proc_outfifo()
{
    int t_stereo = stereo.read();
    int t_stop   = stop.read();

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        OutFifo_State = S_OUTFIFO_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(OutFifo_State.read())
        {
            case S_OUTFIFO_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    OutFifo_State = S_OUTFIFO_DECODE;
                }
                break;

            case S_OUTFIFO_DECODE:
                if (start.read()==(sc_bit)'0')
                {
                    out_fifo( t_stereo );
                    OutFifo_State = S_OUTFIFO_DOWNLOAD;
                }
                break;

            case S_OUTFIFO_DOWNLOAD:
                OutFifo_State = S_OUTFIFO_BUSY;
                break;

            case S_OUTFIFO_BUSY:
                ready = (sc_bit)'1';
                OutFifo_State = S_OUTFIFO_WAIT;
                break;

            default:
                OutFifo_State = S_OUTFIFO_WAIT;
                break;
        }
    }
}

void iii_outfifo::doneThread(){
    cout << "Total sample frames: " << dSampleFrames << endl;
    cout << "Total PCM sample count: " << dPcmCount << endl;
    cout << "Total PCM samples whose values overflowed 14900: " << dPcmOverflowCount << endl;
    delete dWavImage; }

#define BUFSIZE 1
unsigned int outPcm[BUFSIZE];

//////////////////////////////////////////////////////////
// PCM Data write
//
void iii_outfifo::out_fifo( int stereo ) {
    int     ss, sb, ch;
    short pcm;

    if( !dWavImage ) {
        dWavImage = new WavImage();
        dWavImage->Setup(
            stereo+1,                   // # channels
            16,                         // bits/sample
            0,                          // # samples
            44100 );                    // sample rate
        dWavImage->Save( u_mp3_decoder.pcm_filename );
        }
    dSampleFrames++;

    for ( ss=0; ss<SSLIMIT; ss++)
        for ( sb=0; sb<SBLIMIT; sb++)
            for ( ch=0; ch<=stereo; ch++) {
                pcm = u_mp3_decoder.u_decoder_mem->get_pcm_sample( ch, sb, ss);
                dWavImage->Write( pcm );
                dPcmCount++;

                if (pcm > 14900)
                    dPcmOverflowCount++;
                }
}
