///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Subband.cpp
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//

#ifndef SOFT
    extern "C" {
        #include "ilibdd.h"
        #include "iflib_i2s_OMV2_dd.h"
        int __lprintf();
        int __dprintf();
    } 
#endif

#include <math.h>
#include "MP3_Decoder.h"
#include "sc_III_Subband.h"

#include "sc_III_Decoder_Mem.h"

void iii_subband::proc_subband()
{
    int     t_stereo  = stereo.read();

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        Subband_State = S_SUBBAND_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Subband_State.read())
        {
            case S_SUBBAND_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    Subband_State = S_SUBBAND_DECODE;
                }
                break;

            case S_SUBBAND_DECODE:
                if (start.read()==(sc_bit)'0')
                {
                    SubBandSynthesis ( t_stereo);
                    Subband_State = S_SUBBAND_DOWNLOAD;
                }
                break;

            case S_SUBBAND_DOWNLOAD:
                Subband_State = S_SUBBAND_BUSY;
#ifdef _DEBUG_OUT
                {
                    FILE *fp;
                    int ss, sb, ch;

                    fp = fopen("dbg_subband.txt", "a+");

                    fprintf(fp, "Frame=%d/Granule=%d/Channel=%d\n",
                                    u_mp3_decoder.u_decoder_ctl->nFrame,
                                    ((u_mp3_decoder.u_decoder_ctl->gr)? 1:0),
                                    ((u_mp3_decoder.u_decoder_ctl->ch)? 1:0) );

                    for (ss=0; ss<SSLIMIT; ss++)
                        for (sb=0; sb<SBLIMIT; sb++)
                            for (ch=0; ch<=t_stereo; ch++)
                                fprintf(fp, "[%d][%2d][%2d] %8d\n", ch, sb, ss,
                                            u_mp3_decoder.u_decoder_mem->get_pcm_sample(ch, sb, ss));
                    fflush(fp);
                    fclose(fp);
                }
#endif
                break;

            case S_SUBBAND_BUSY:
                ready = (sc_bit)'1';
                Subband_State = S_SUBBAND_WAIT;
                break;

            default:
                Subband_State = S_SUBBAND_WAIT;
                break;
        }
    }
}

///////////////////////////////////////////////////////////////////////////
// Polyphase-Subband Filter
//
void iii_subband::SubBandSynthesis (int stereo)
{
    int         ch, ss, sb, i, j, k;
    int         w_sign, w_k;
    double      *bufOffsetPtr, sum;
    long        foo;

/*
FILE *fpw;
fpw=fopen("xxx.dat","a+t");
*/

    static int  bufOffset[2] = {64,64};

    // Synthesis Filter Window Coeff /////////////////////////////////////////////
    static double syn_win[256] = {
         0.0000000000,  -0.0000152590,  -0.0000152590,  -0.0000152590,
        -0.0000152590,  -0.0000152590,  -0.0000152590,  -0.0000305180,
        -0.0000305180,  -0.0000305180,  -0.0000305180,  -0.0000457760,
        -0.0000457760,  -0.0000610350,  -0.0000610350,  -0.0000762940,
        -0.0000762940,  -0.0000915530,  -0.0001068120,  -0.0001068120,
        -0.0001220700,  -0.0001373290,  -0.0001525880,  -0.0001678470,
        -0.0001983640,  -0.0002136230,  -0.0002441410,  -0.0002593990,
        -0.0002899170,  -0.0003204350,  -0.0003662110,  -0.0003967290,
        -0.0004425050,  -0.0004730220,  -0.0005340580,  -0.0005798340,
        -0.0006256100,  -0.0006866460,  -0.0007476810,  -0.0008087160,
        -0.0008850100,  -0.0009613040,  -0.0010375980,  -0.0011138920,
        -0.0012054440,  -0.0012969970,  -0.0013885500,  -0.0014801030,
        -0.0015869140,  -0.0016937260,  -0.0017852780,  -0.0019073490,
        -0.0020141600,  -0.0021209720,  -0.0022430420,  -0.0023498540,
        -0.0024566650,  -0.0025787350,  -0.0026855470,  -0.0027923580,
        -0.0028991700,  -0.0029907230,  -0.0030822750,  -0.0031738280,
         0.0032501220,   0.0033264160,   0.0033874510,   0.0034332280,
         0.0034637450,   0.0034790040,   0.0034790040,   0.0034637450,
         0.0034179690,   0.0033721920,   0.0032806400,   0.0031738280,
         0.0030517580,   0.0028839110,   0.0027008060,   0.0024871830,
         0.0022277830,   0.0019378660,   0.0016174320,   0.0012664790,
         0.0008697510,   0.0004425050,  -0.0000305180,  -0.0005493160,
        -0.0010986330,  -0.0016937260,  -0.0023345950,  -0.0030059810,
        -0.0037231450,  -0.0044860840,  -0.0052948000,  -0.0061187740,
        -0.0070037840,  -0.0079193120,  -0.0088653560,  -0.0098419190,
        -0.0108489990,  -0.0118865970,  -0.0129394530,  -0.0140228270,
        -0.0151214600,  -0.0162353520,  -0.0173492430,  -0.0184631350,
        -0.0195770260,  -0.0206909180,  -0.0217895510,  -0.0228576660,
        -0.0239105220,  -0.0249328610,  -0.0259094240,  -0.0268402100,
        -0.0277252200,  -0.0285339360,  -0.0292816160,  -0.0299377440,
        -0.0305328370,  -0.0310058590,  -0.0313873290,  -0.0316619870,
        -0.0318145750,  -0.0318450930,  -0.0317382810,  -0.0314788820,
         0.0310821530,   0.0305175780,   0.0297851560,   0.0288848880,
         0.0278015140,   0.0265350340,   0.0250854490,   0.0234222410,
         0.0215759280,   0.0195312500,   0.0172576900,   0.0148010250,
         0.0121154790,   0.0092315670,   0.0061340330,   0.0028228760,
        -0.0006866460,  -0.0043945310,  -0.0083160400,  -0.0124206540,
        -0.0167083740,  -0.0211791990,  -0.0258178710,  -0.0306091310,
        -0.0355529790,  -0.0406341550,  -0.0458374020,  -0.0511322020,
        -0.0565338130,  -0.0619964600,  -0.0675201420,  -0.0730590820,
        -0.0786285400,  -0.0841827390,  -0.0897064210,  -0.0951690670,
        -0.1005401610,  -0.1058197020,  -0.1109466550,  -0.1159210210,
        -0.1206970210,  -0.1252593990,  -0.1295623780,  -0.1335906980,
        -0.1372985840,  -0.1406707760,  -0.1436767580,  -0.1462554930,
        -0.1484222410,  -0.1501159670,  -0.1513061520,  -0.1519622800,
        -0.1520690920,  -0.1515960690,  -0.1504974370,  -0.1487731930,
        -0.1463623050,  -0.1432647710,  -0.1394500730,  -0.1348876950,
        -0.1295776370,  -0.1234741210,  -0.1165771480,  -0.1088562010,
         0.1003112790,   0.0909271240,   0.0806884770,   0.0695953370,
         0.0576171870,   0.0447845460,   0.0310821530,   0.0165100100,
         0.0010681150,  -0.0152282710,  -0.0323791500,  -0.0503540040,
        -0.0691680910,  -0.0887756350,  -0.1091613770,  -0.1303100590,
        -0.1522064210,  -0.1747894290,  -0.1980590820,  -0.2219848630,
        -0.2465057370,  -0.2715911870,  -0.2972106930,  -0.3233184810,
        -0.3498687740,  -0.3768005370,  -0.4040832520,  -0.4316558840,
        -0.4594726560,  -0.4874725340,  -0.5156097410,  -0.5438232420,
        -0.5720367430,  -0.6002197270,  -0.6282958980,  -0.6562194820,
        -0.6839141850,  -0.7113189700,  -0.7383728030,  -0.7650299070,
        -0.7912139890,  -0.8168640140,  -0.8419494630,  -0.8663635250,
        -0.8900909420,  -0.9130554200,  -0.9351959230,  -0.9564819340,
        -0.9768524170,  -0.9962463380,  -1.0146179200,  -1.0319366460,
        -1.0481567380,  -1.0632171630,  -1.0771179200,  -1.0897827150,
        -1.1012115480,  -1.1113739010,  -1.1202239990,  -1.1277465820,
        -1.1339263920,  -1.1387634280,  -1.1422119140,  -1.1442871090
    };

    for (ch=0; ch<=stereo; ch++)
    {
        for(ss=0; ss<SSLIMIT; ss++)
        {
#ifdef OPTIMIZE

            bufOffset[ch] = (bufOffset[ch]-64) & 0x3ff;
            bufOffsetPtr = &((*buf)[ch][bufOffset[ch]]);

            #define cos1_64  0.500602998235196
            #define cos3_64  0.505470959897544
            #define cos5_64  0.515447309922625
            #define cos7_64  0.531042591089784
            #define cos9_64  0.553103896034445
            #define cos11_64 0.582934968206134
            #define cos13_64 0.622504123035665
            #define cos15_64 0.674808341455006
            #define cos17_64 0.744536271002299
            #define cos19_64 0.839349645415527
            #define cos21_64 0.972568237861961
            #define cos23_64 1.169439933432885
            #define cos25_64 1.484164616314166
            #define cos27_64 2.057781009953411
            #define cos29_64 3.407608418468719
            #define cos31_64 10.190008123548033
            #define cos1_32  0.502419286188156
            #define cos3_32  0.522498614939689
            #define cos5_32  0.566944034816358
            #define cos7_32  0.646821783359990
            #define cos9_32  0.788154623451250
            #define cos11_32 1.060677685990347
            #define cos13_32 1.722447098238334
            #define cos15_32 5.101148618689155
            #define cos1_16  0.509795579104159
            #define cos3_16  0.601344886935045
            #define cos5_16  0.899976223136416
            #define cos7_16  2.562915447741505
            #define cos1_8   0.541196100146197
            #define cos3_8   1.306562964876376
            #define cos1_4   0.707106781186547

          {
            int i;
            register double tmp;
            double p00,p01,p02,p03,p04,p05,p06,p07,p08,p09,p10,p11,p12,p13,p14,p15;
            double q00,q01,q02,q03,q04,q05,q06,q07,q08,q09,q10,q11,q12,q13,q14,q15;

            p00 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 0,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,31,ss);
            p01 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 1,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,30,ss);
            p02 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 2,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,29,ss);
            p03 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 3,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,28,ss);
            p04 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 4,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,27,ss);
            p05 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 5,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,26,ss);
            p06 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 6,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,25,ss);
            p07 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 7,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,24,ss);
            p08 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 8,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,23,ss);
            p09 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 9,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,22,ss);
            p10 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,10,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,21,ss);
            p11 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,11,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,20,ss);
            p12 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,12,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,19,ss);
            p13 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,13,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,18,ss);
            p14 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,14,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,17,ss);
            p15 = u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,15,ss) 
                + u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,16,ss);

            q00 = p00 + p15;
            q01 = p01 + p14;
            q02 = p02 + p13;
            q03 = p03 + p12;
            q04 = p04 + p11;
            q05 = p05 + p10;
            q06 = p06 + p09;
            q07 = p07 + p08;
            q08 = cos1_32  * (p00 + p15);
            q09 = cos3_32  * (p01 + p14);
            q10 = cos5_32  * (p02 + p13);
            q11 = cos7_32  * (p03 + p12);
            q12 = cos9_32  * (p04 + p11);
            q13 = cos11_32 * (p05 + p10);
            q14 = cos13_32 * (p06 + p09);
            q15 = cos15_32 * (p07 + p08);
    
            p00 = q00 + q07;
            p01 = q01 + q06;
            p02 = q02 + q05;
            p03 = q03 + q04;
            p04 = cos1_16 * (q00 - q07);
            p05 = cos3_16 * (q01 - q06);
            p06 = cos5_16 * (q02 - q05);
            p07 = cos7_16 * (q03 - q04);
            p08 = q08 + q15;
            p09 = q09 + q14;
            p10 = q10 + q13;
            p11 = q11 + q12;
            p12 = cos1_16 * (q08 - q15);
            p13 = cos3_16 * (q09 - q14);
            p14 = cos5_16 * (q10 - q13);
            p15 = cos7_16 * (q11 - q12);
        
            q00 = p00 + p03;
            q01 = p01 + p02;
            q02 = cos1_8 * (p00 - p03);
            q03 = cos3_8 * (p01 - p02);
            q04 = p04 + p07;
            q05 = p05 + p06;
            q06 = cos1_8 * (p04 - p07);
            q07 = cos3_8 * (p05 - p06);
            q08 = p08 + p11;
            q09 = p09 + p10;
            q10 = cos1_8 * (p08 - p11);
            q11 = cos3_8 * (p09 - p10);
            q12 = p12 + p15;
            q13 = p13 + p14;
            q14 = cos1_8 * (p12 - p15);
            q15 = cos3_8 * (p13 - p14); 
        
            p00 = q00 + q01;
            p01 = cos1_4 * (q00 - q01);
            p02 = q02 + q03;
            p03 = cos1_4 * (q02 - q03);
            p04 = q04 + q05;
            p05 = cos1_4 * (q04 - q05);
            p06 = q06 + q07;
            p07 = cos1_4 * (q06 - q07);
            p08 = q08 + q09;
            p09 = cos1_4 * (q08 - q09);
            p10 = q10 + q11;
            p11 = cos1_4 * (q10 - q11);
            p12 = q12 + q13;
            p13 = cos1_4 * (q12 - q13);
            p14 = q14 + q15;
            p15 = cos1_4 * (q14 - q15);

            tmp              = p06 + p07;
            bufOffsetPtr[36] = -(p05 + tmp);
            bufOffsetPtr[44] = -(p04 + tmp);
            tmp              = p11 + p15;
            bufOffsetPtr[10] = tmp;
            bufOffsetPtr[6]  = p13 + tmp;
            tmp              = p14 + p15;
            bufOffsetPtr[46] = -(p08  + p12 + tmp);
            bufOffsetPtr[34] = -(p09  + p13 + tmp);
            tmp             += p10 + p11;
            bufOffsetPtr[38] = -(p13 + tmp);
            bufOffsetPtr[42] = -(p12 + tmp);
            bufOffsetPtr[2]  = p09 + p13 + p15;
            bufOffsetPtr[4]  = p05 + p07;
            bufOffsetPtr[48] = -p00;
            bufOffsetPtr[0]  = p01;
            bufOffsetPtr[8]  = p03;
            bufOffsetPtr[12] = p07;
            bufOffsetPtr[14] = p15;
            bufOffsetPtr[40] = -(p02 + p03);
            
            p00 = cos1_64  * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 0,ss) 
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,31,ss));
            p01 = cos3_64  * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 1,ss)
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,30,ss));
            p02 = cos5_64  * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 2,ss) 
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,29,ss));
            p03 = cos7_64  * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 3,ss) 
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,28,ss));
            p04 = cos9_64  * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 4,ss) 
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,27,ss));
            p05 = cos11_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 5,ss) 
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,26,ss));
            p06 = cos13_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 6,ss) 
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,25,ss));
            p07 = cos15_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 7,ss) 
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,24,ss));
            p08 = cos17_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 8,ss) 
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,23,ss));
            p09 = cos19_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch, 9,ss) 
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,22,ss));
            p10 = cos21_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,10,ss)
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,21,ss));
            p11 = cos23_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,11,ss)
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,20,ss));
            p12 = cos25_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,12,ss)
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,19,ss));
            p13 = cos27_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,13,ss)
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,18,ss));
            p14 = cos29_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,14,ss)
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,17,ss));
            p15 = cos31_64 * (u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,15,ss)
                            - u_mp3_decoder.u_decoder_mem->get_hybridOut(ch,16,ss));
        
            q00 = p00 + p15;
            q01 = p01 + p14;
            q02 = p02 + p13;
            q03 = p03 + p12;
            q04 = p04 + p11;
            q05 = p05 + p10;
            q06 = p06 + p09;
            q07 = p07 + p08;
            q08 = cos1_32  * (p00 - p15);
            q09 = cos3_32  * (p01 - p14);
            q10 = cos5_32  * (p02 - p13);
            q11 = cos7_32  * (p03 - p12);
            q12 = cos9_32  * (p04 - p11);
            q13 = cos11_32 * (p05 - p10);
            q14 = cos13_32 * (p06 - p09);
            q15 = cos15_32 * (p07 - p08);

            p00 = q00 + q07;
            p01 = q01 + q06;
            p02 = q02 + q05;
            p03 = q03 + q04;
            p04 = cos1_16 * (q00 - q07);
            p05 = cos3_16 * (q01 - q06);
            p06 = cos5_16 * (q02 - q05);
            p07 = cos7_16 * (q03 - q04);
            p08 = q08 + q15;
            p09 = q09 + q14;
            p10 = q10 + q13;
            p11 = q11 + q12;
            p12 = cos1_16 * (q08 - q15);
            p13 = cos3_16 * (q09 - q14);
            p14 = cos5_16 * (q10 - q13);
            p15 = cos7_16 * (q11 - q12);

            q00 = p00 + p03;
            q01 = p01 + p02;
            q02 = cos1_8 * (p00 - p03);
            q03 = cos3_8 * (p01 - p02);
            q04 = p04 + p07;
            q05 = p05 + p06;
            q06 = cos1_8 * (p04 - p07);
            q07 = cos3_8 * (p05 - p06);
            q08 = p08 + p11;
            q09 = p09 + p10;
            q10 = cos1_8 * (p08 - p11);
            q11 = cos3_8 * (p09 - p10);
            q12 = p12 + p15;
            q13 = p13 + p14;
            q14 = cos1_8 * (p12 - p15);
            q15 = cos3_8 * (p13 - p14);

            p00 = q00 + q01;
            p01 = cos1_4 * (q00 - q01);
            p02 = q02 + q03;
            p03 = cos1_4 * (q02 - q03);
            p04 = q04 + q05;
            p05 = cos1_4 * (q04 - q05);
            p06 = q06 + q07;
            p07 = cos1_4 * (q06 - q07);
            p08 = q08 + q09;
            p09 = cos1_4 * (q08 - q09);
            p10 = q10 + q11;
            p11 = cos1_4 * (q10 - q11);
            p12 = q12 + q13;
            p13 = cos1_4 * (q12 - q13);
            p14 = q14 + q15;
            p15 = cos1_4 * (q14 - q15);

            tmp              = p13 + p15;
            bufOffsetPtr[1]  = p01 + p09 + tmp;
            bufOffsetPtr[5]  = p05 + p07 + p11 + tmp;
            tmp             += p09;
            bufOffsetPtr[33] = -(p01 + p14 + tmp);
            tmp             += p05 + p07;
            bufOffsetPtr[3]  = tmp;
            bufOffsetPtr[35] = -(p06 + p14 + tmp);
            tmp              = p10 + p11 + p12 + p13 + p14 + p15;
            bufOffsetPtr[39] = -(p02 + p03 + tmp - p12);
            bufOffsetPtr[43] = -(p04 + p06 + p07 + tmp - p13);
            bufOffsetPtr[37] = -(p05 + p06 + p07 + tmp - p12);
            bufOffsetPtr[41] = -(p02 + p03 + tmp - p13);
            tmp              = p08 + p12 + p14 + p15;
            bufOffsetPtr[47] = -(p00 + tmp);
            bufOffsetPtr[45] = -(p04 + p06 + p07 + tmp);
            tmp              = p11 + p15;
            bufOffsetPtr[11] = p07 + tmp;
            tmp             += p03;
            bufOffsetPtr[9]  = tmp;
            bufOffsetPtr[7]  = p13 + tmp;
            bufOffsetPtr[13] = p07 + p15;
            bufOffsetPtr[15] = p15;

            bufOffsetPtr[16] = 0.0;
            for (i = 0; i < 16; i++)
            {
                bufOffsetPtr[32-i] = -bufOffsetPtr[i];
                bufOffsetPtr[63-i] = bufOffsetPtr[33+i]; 
            }
          }
          {
            double *dt[16];

            for(i=0; i<16; i++)
            {
                k=(i<<5);
                dt[i]=(*buf)[ch] +(((k+(((i+1)>>1)<<6)) + bufOffset[ch])&0x3ff);
            }
// fprintf(fpw,"*");
            for(j=0; j<32; j++)
            {
                sum=
                    syn_win[j      ] * dt[ 0][j]+
                    syn_win[j +  32] * dt[ 1][j]+
                    syn_win[j +  64] * dt[ 2][j]+
                    syn_win[j +  96] * dt[ 3][j]+
                    syn_win[j + 128] * dt[ 4][j]+
                    syn_win[j + 160] * dt[ 5][j]+
                    syn_win[j + 192] * dt[ 6][j]+
                    syn_win[j + 224] * dt[ 7][j]+
                    syn_win[j + 256] * dt[ 8][j]+
                    syn_win[j + 288] * dt[ 9][j]+
                    syn_win[j + 320] * dt[10][j]+
                    syn_win[j + 352] * dt[11][j]+
                    syn_win[j + 384] * dt[12][j]+
                    syn_win[j + 416] * dt[13][j]+
                    syn_win[j + 448] * dt[14][j]+
                    syn_win[j + 480] * dt[15][j];

                if(sum>0)
                    foo=(long)(sum * (double)32768 + (double)0.5);
                else
                    foo=(long)(sum * (double)32768 - (double)0.5);

                if(foo >= (long)32768)
                    u_mp3_decoder.u_decoder_mem->put_pcm_sample( (short)32767, ch, j, ss);
                else if (foo < (long) -32768)
                    u_mp3_decoder.u_decoder_mem->put_pcm_sample( (short)-32768, ch, j, ss);
                else
                    u_mp3_decoder.u_decoder_mem->put_pcm_sample( (short)foo, ch, j, ss);

/*
if(foo >= (long)32768)
    fprintf(fpw,"%d,", (short)32767);
else if (foo < (long) -32768)
    fprintf(fpw,"%d,", (short)-32767);
else
    fprintf(fpw,"%d,", (short)foo);
*/

            }
// fprintf(fpw,"\n");
          }

#else
            // Subband Synthesis //////////////////////
            bufOffset[ch] = (bufOffset[ch] - 64) & 0x3ff;
            bufOffsetPtr = &((*buf)[ch][bufOffset[ch]]);

            // Filter /////////////////////////////////
            for (i=0; i<64; i++)
            {
                sum = 0;
                for (sb=0; sb<32; sb++)
                    sum += u_mp3_decoder.u_decoder_mem->get_hybridOut( ch, sb, ss) * (*filter)[i][sb];
                bufOffsetPtr[i] = sum;
            }

            // De-Windowing ///////////////////////////
// fprintf(fpw,"#");
            for (sb=0; sb<32; sb++)
            {
                sum = 0;
                for (i=0; i<16; i++)
                {
                    k = sb + (i<<5);

                    w_sign = (k>255)? -1 : 1;
                    w_k = (k>255)? (512-k-1) : k;

                    sum += (double)w_sign * syn_win[w_k]
                                * (*buf)[ch] [( (k + ( ((i+1)>>1) <<6) ) + bufOffset[ch]) & 0x3ff];
                }

                // Truncation ////////////////////////////////////////////
                if(sum > 0)
                    foo = (long)(sum * (double)32768 + (double)0.5);
                else
                    foo = (long)(sum * (double)32768 - (double)0.5);

                // Save PCM Outputs ///////////////////////////////////////
                if (foo >= (long)32768)
                    u_mp3_decoder.u_decoder_mem->put_pcm_sample( (short)32767, ch, sb, ss);
                else if (foo < (long) -32768)
                    u_mp3_decoder.u_decoder_mem->put_pcm_sample( (short)-32768, ch, sb, ss);
                else
                    u_mp3_decoder.u_decoder_mem->put_pcm_sample( (short)foo, ch, sb, ss);

/*
if(foo >= (long)32768)
    fprintf(fpw,"%d,", (short)32767);
else if (foo < (long) -32768)
    fprintf(fpw,"%d,", (short)-32767);
else
    fprintf(fpw,"%d,", (short)foo);
*/

            }   // SUBBAND
// fprintf(fpw,"\n");
#endif
        }   // SUBSAMPLE
    }   // CHANNEL
// fclose(fpw);
}


void iii_subband::create_syn_filter(double filter[64][SBLIMIT])
{
    int i,k;
    
    for (i=0; i<64; i++)
        for (k=0; k<32; k++)
        {
            if ((filter[i][k] = 1e9*cos((double)((PI64*(double)i+PI4)*(2*(double)k+1)))) >= 0)
#ifdef SOFT
                modf(filter[i][k]+0.5, &filter[i][k]);
            else
                modf(filter[i][k]-0.5, &filter[i][k]);
#else
                filter[i][k] = floor(filter[i][k]+0.5);
            else
                filter[i][k] = floor(filter[i][k]-0.5);
#endif
            filter[i][k] *= 1e-9;
        }
}
