///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Subband.h
// Project  : MP3 Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision :
// Comment  :
//
//
#ifndef III_SUBBAND_INC
#define III_SUBBAND_INC

#include "systemc.h"

enum tSubband_State {
    S_SUBBAND_WAIT,
    S_SUBBAND_DECODE,
    S_SUBBAND_DOWNLOAD,
    S_SUBBAND_BUSY };


SC_MODULE(iii_subband)
{
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Subband Start
    sc_out<sc_bit>      ready;  // Subband Ready

    sc_in<sc_bit>       stereo;

    // Process ///////////////////////////////////////////////////////////
    void proc_subband();

    // Signals ///////////////////////////////////////////////////////////
    sc_signal<tSubband_State>   Subband_State;  // FSM State Variable

    // Functions /////////////////////////////////////////////////////////
    void SubBandSynthesis ( int stereo);
    void create_syn_filter(double filter[64][SBLIMIT]);

    // Coefficients //////////////////////////////////////////////////////
    #ifndef PI
    #define PI      3.14159265358979
    #endif

    #define PI4     PI/4.0
    #define PI64    PI/64.0

    // Variables /////////////////////////////////////////////////////////
    typedef double  NN[64][32];
    NN *filter;
    typedef double  BB[2][1024];
    BB *buf;

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(iii_subband)
    {
        SC_METHOD(proc_subband);
        sensitive << clk << rst;

        // Create Synthesis Filter Coefficients //////////////////////////////
        {
            buf    = (BB *)new(BB);
#ifndef OPTIMIZE
            filter = (NN *)new(NN);
            create_syn_filter(*filter);
#endif
        }

    }
};

#endif
