///////////////////////////////////////////////////////////////////////////
// Filename : FrameBuffer.cxx
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//
 
#include "FrameBuffer.hxx"

//===========================================================================
// FrameBuffer                                                  johnS 4-21-01
//===========================================================================

FrameBuffer  *GlobalFrameBuffer;
 
//---------------------------------------------------------
// ::clockThread() - Extract new header info from each
//                   MP3 frame, set data up in frame
//                   buffer memory
//
// Description                                johnS 4-21-01
// -----------
//---------------------------------------------------------

void FrameBuffer::clockThread(){
    if( Reset.read() == (sc_bit)'0' ){
        HdrReady = '1';
        dState = S_FILL_BUFF_WAIT;
        dFrameBytesStart = 0;
        }
    else if( Clock.read() == (sc_bit)'1' )
        switch( dState ){
            case S_FILL_BUFF_WAIT:
                if (FillStream.read()==(sc_bit)'1' ){
                    StreamFull = (sc_bit)'1';
                    RequestFrameOut();
                    HdrReady = '0';
                    dState = S_HEADER_WAIT;
                    }
                break;

            case S_HEADER_WAIT:
                if( HdrStart.read() == (sc_bit)'1' ){
                    StreamFull = (sc_bit)'0';
                    dState = S_HEADER_DECODE;
                    }
                break;

            case S_HEADER_DECODE:
                if( HdrStart.read()==(sc_bit)'0' ){
                    dMp3Header = FrameHeaderIn;
                    FrameHeader = dMp3Header->FrameHeader;
                    getDataBuffer();
                    selectSideHeaderThread();
                    dState = S_HEADER_DOWNLOAD;
                    }
                break;
 
            case S_HEADER_DOWNLOAD:
//printf( "ReadID:  %8d WriteID: %8d TotBits: %8d CRC:     %8x\n",
//dBufferReadIndex, dBufferWriteIndex, dBufferTotalBits, dCrc );

                dState = S_HEADER_BUSY; break;
 
            case S_HEADER_BUSY:
                HdrReady = '1';
                dState = S_FILL_BUFF_WAIT; break;
 
            default:
                dState = S_HEADER_WAIT; break;
            }
}
 
//---------------------------------------------------------
// ::selectSideHeaderThread()
//
// Description                                johnS 4-21-01
// -----------
// Select approriate side header from MP3 header block based
// on current channel and granule selector coming from
// decoder ctl module.
//---------------------------------------------------------

void FrameBuffer::selectSideHeaderThread(){
    int ch = Channel.read();
    int gr = Granule.read();
    if( dMp3Header ){
        ScfsiBits  = dMp3Header->SideHeaderBlock.Ch[ch].ScfsiBits;
        SideHeader = &dMp3Header->SideHeaderBlock.Ch[ch].Gr[gr];
        }
}
 
//---------------------------------------------------------
// ::dataBufferInThread()
//       - Accept new main data slot buffer from Mp3 stream
//
// Description                                johnS 4-21-01
// -----------
// Accept new main data slot buffer, transfer into local
// circular buffer in preparation to service frame buffer
// requests from decoder core modules.
//---------------------------------------------------------
void FrameBuffer::getDataBuffer(){
    const Mp3DataBuffer *dataBufferIn  = DataBufferIn;
    int nSlots = dataBufferIn->WriteIndex;

    // Transfer contents into local circular buffer.
    for( int i=0; i<nSlots; i++ )
        putBuffer( dataBufferIn->Buffer[i] );

    // Align to end position of previous frame ...
    unsigned mainDataEnd = dBufferTotalBits / 8;
    unsigned alignBits   = dBufferTotalBits % 8;
 
    if( alignBits ){ // Layer III : BitperSlot=8
        GetBits( 8 - alignBits );
        mainDataEnd++;
        }

    unsigned bytesToDiscard = dFrameBytesStart
        - mainDataEnd - dMp3Header->SideHeaderBlock.MainDataBegin;
 
    dFrameBytesStart += nSlots;
    for( ; bytesToDiscard > 0; bytesToDiscard-- )
        GetBits(8);
}
