///////////////////////////////////////////////////////////////////////////
// Filename : FrameBuffer.hxx
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

//===========================================================================
// FrameBuffer                                                  johnS 4-23-01
//===========================================================================

#include <systemc.h>
#include "Mp3Common.h"

extern "C" {
extern unsigned long updateCrc( int ch, unsigned long CRC );
};
 
SC_MODULE( FrameBuffer ){

    //-------------------------------------------------------
    // Untimed port declarations
 
    sc_master<>                        RequestFrameOut; // --> master
    sc_inmaster<const Mp3Header *>     FrameHeaderIn;   // <-- inmaster
    sc_inmaster<const Mp3DataBuffer *> DataBufferIn;    // <-- inmaster

    //-------------------------------------------------------
    // Timed port declarations
    sc_in_clk      Clock;
    sc_in<sc_bit>  Reset;

    sc_in<sc_bit>  FillStream;
    sc_out<sc_bit> StreamFull;

    sc_in<sc_bit>  HdrStart;
    sc_out<sc_bit> HdrReady;

    sc_in<sc_bit>  Channel;
    sc_in<sc_bit>  Granule;

    sc_out<Mp3FrameHeader>        FrameHeader;
    sc_out<const Mp3SideHeader *> SideHeader;
    sc_out<unsigned>              ScfsiBits;

    //-------------------------------------------------------
    // Context declarations
    private:
        enum FrameBufferState {
            S_FILL_BUFF_WAIT,
            S_HEADER_WAIT,
            S_HEADER_DECODE,
            S_HEADER_DOWNLOAD,
            S_HEADER_BUSY };

        FrameBufferState dState;
        const Mp3Header *dMp3Header;

        unsigned char dBuffer[BUFFER_SIZE];
        unsigned dBufferReadIndex;
        unsigned dBufferWriteIndex;
        unsigned long dBufferTotalBits;
        int dBufferBitIndex;
        int dFrameBytesStart;
 
    //-------------------------------------------------------
    // Helper declarations
    private:
        void putBuffer( unsigned char byte ){
            dBuffer[dBufferWriteIndex%BUFFER_SIZE] = byte;
            dBufferWriteIndex++;
            }
 
    //-------------------------------------------------------
    // Public FrameBuffer Memory Accessors
 
    public:
unsigned long dCrc;
        unsigned long GetBits( int range ) {
            register unsigned long data = 0;
            register unsigned char ch = dBuffer[dBufferReadIndex];
 
            for( int i=0; i<range; i++ ){
                data <<= 1;
                data |= (ch >> dBufferBitIndex) & 1;
 
                // if( current byte is exhausted ) ...
                if( --dBufferBitIndex < 0 ){
                    // if( check for wraparound of circular buffer ) ...
                    if( ++dBufferReadIndex == BUFFER_SIZE )
                        dBufferReadIndex = 0;
                    ch = dBuffer[dBufferReadIndex];
//dCrc = updateCrc( ch, dCrc );
                    dBufferBitIndex = 7;
                    }
                }
            dBufferTotalBits += range;
            return data;
            }

        unsigned long TellBits(){ return dBufferTotalBits; }
        void RewindBits( int rewindBits ){
            dBufferTotalBits -= rewindBits;
            dBufferBitIndex += rewindBits;

            while( dBufferBitIndex >= 8 ){
                dBufferBitIndex -= 8;
                dBufferReadIndex--;
                }
            dBufferReadIndex = dBufferReadIndex % BUFFER_SIZE;
            }

    //-------------------------------------------------------
    // Thread declarations
    private:
        void clockThread();
        void selectSideHeaderThread();
        void getDataBuffer();

    public:
    SC_CTOR( FrameBuffer )
        : dState(S_FILL_BUFF_WAIT), dMp3Header(NULL),
          dBufferReadIndex(0), dBufferWriteIndex(0),
          dBufferTotalBits(0), dBufferBitIndex(7),
          dFrameBytesStart(0)
    {
        //--------------------------------------
        // Thread bindings
dCrc = 0xffffffffL;

        SC_METHOD( clockThread );
        sensitive << Clock;

        SC_METHOD( selectSideHeaderThread );
        sensitive << Channel << Granule;
        }
    };
 
extern FrameBuffer *GlobalFrameBuffer;
