///////////////////////////////////////////////////////////////////////////
// Filename : PcmStream.hxx
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

//===========================================================================
// PcmStream                                                    johnS 4-21-01
//===========================================================================
 
#include <systemc.h>
#include <string>
#include "Mp3Common.h"

class WavImage;

SC_MODULE( PcmStream ){

    //-------------------------------------------------------
    // Abstract port declarations
    sc_inslave<const PcmFileInfo *>    FileHeaderIn;
    sc_inslave<const PcmSampleFrame *> PcmFrameIn;
    sc_slave<>                 DoneIn;

    //-------------------------------------------------------
    // Context declarations
    private:
        WavImage *dWavImage;
        string dFileName;
        int dIsStereo;
        unsigned long dSampleFrames, dPcmCount, dPcmOverflowCount;
 
    //-------------------------------------------------------
    // Thread declarations
    private:
 
        void fileHeaderInThread();
        void pcmFrameInThread();
        void doneInThread();

    //-------------------------------------------------------
    // Accessors
    public:
        void SetFileBaseName( const char *fileBaseName ){
            dFileName = fileBaseName; }
 
    //-------------------------------------------------------
    // Helper declarations
    private:

    public:
        SC_CTOR( PcmStream )
            : dWavImage(NULL),
              dIsStereo(0),
              dSampleFrames(0), dPcmCount(0), dPcmOverflowCount(0) {

            //--------------------------------------
            // Thread bindings

            SC_SLAVE( fileHeaderInThread, FileHeaderIn );
            SC_SLAVE( pcmFrameInThread,   PcmFrameIn );
            SC_SLAVE( doneInThread,       DoneIn );
            }
    };
