//===========================================================================
// PlayControl                                                  johnS 4-21-01
//
// The PlayControl module provides a simple interface to start and stop
// an MP3 file decode and playback process.
//
// The Play and Stop outputs are driven from autonomous threads that can
// be coupled say over a socket to some external GUI interface.
//
// The Done input is confirmation that the decoder hardware has acknowledged
// a Stop command or that the input stream has been fully processed which
// implies a Stop command.
//===========================================================================

SC_MODULE( PlayControl ){

    //-------------------------------------------------------
    // Abstract port declarations
    sc_master<> PlayOut;
    sc_master<> StopOut;
    sc_slave<>  EosIn;
    sc_slave<>  DoneIn;
    sc_master<> ClosePcm;
 
    //-------------------------------------------------------
    // Thread declarations
    private:
        void playOutThread(){ // Autonomous Play button interface thread
            // For now we just kick off the simulation from the start.
            PlayOut();

            // This thread is the hook to the 'Play' button of a
            // future GUI interface.  The interface code would look
            // something like this:
            //
            // while( !<external interface stop detect> );
            // StopOut();
            //
            }

        void eosInThread(){ StopOut(); }

        void doneInThread(){ // Done slave thread
            ClosePcm();
            cout << "... MP3 decode and playback done !\n" << endl;
            sc_stop();
            }

    //-------------------------------------------------------
    // Constructor
    public:
    SC_CTOR( PlayControl ) {
        //--------------------------------------
        // Thread bindings
        SC_THREAD( playOutThread );
        sensitive << UTick;

        SC_SLAVE( eosInThread, EosIn );
        SC_SLAVE( doneInThread, DoneIn );
        }
    };
