///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Decoder_ctl.cpp
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//

#include "sc_III_Decoder_ctl.h"

void decoder_ctl::proc_decoder_ctl()
{
    // Decoder Ctl FSM //////////////////////////////////////
    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        nFrame = 0;

        S_III_Decoder   = S_STOP_WAIT;
        hdr_start       = (sc_bit)'0';
        scl_start       = (sc_bit)'0';
        huff_start      = (sc_bit)'0';
        deq_start       = (sc_bit)'0';
        stereo_start    = (sc_bit)'0';
        antialias_start = (sc_bit)'0';
        hybrid_start    = (sc_bit)'0';
        subband_start   = (sc_bit)'0';

        ch = false;
        gr = false;

        mem_init = (sc_bit)'0';
        fill_stream = (sc_bit)'0';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(S_III_Decoder.read())
        {
            case S_STOP_WAIT:
                if( dIsStopped == 0 ) { // Stop Button
                    mem_init = (sc_bit)'1';
                    S_III_Decoder = S_INIT_MEM;
                }
                break;

            case S_INIT_MEM:
                if (m_init_done.read()==(sc_bit)'1') {
                    mem_init = (sc_bit)'0';
                    S_III_Decoder = S_READ_MP3;
                }
                break;

            case S_READ_MP3:
                fill_stream = (sc_bit)'1';
                S_III_Decoder = S_WAIT_FILL_STREAM;
                break;

            case S_WAIT_FILL_STREAM:
                if (stream_full.read()==(sc_bit)'1') {
                    fill_stream = (sc_bit)'0';
                    ch = false;
                    gr = false;
                    hdr_start = (sc_bit)'1';
                    S_III_Decoder = S_START_HEADER;
                }
                break;

            case S_START_HEADER:
                if (hdr_ready.read()==(sc_bit)'0') {
                    hdr_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_HEADER;
                }
                break;

            case S_WAIT_HEADER:
                if (hdr_ready.read()==(sc_bit)'1') {
                    scl_start = (sc_bit)'1';
                    S_III_Decoder = S_START_SCALE;
                }
                break;

            case S_START_SCALE:
                if (scl_ready.read()==(sc_bit)'0') {
                    scl_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_SCALE;
                }
                break;

            case S_WAIT_SCALE:
                if (scl_ready.read()==(sc_bit)'1') {
                    huff_start = (sc_bit)'1';
                    S_III_Decoder = S_START_HUFF;
                }
                break;

            case S_START_HUFF:
                if (huff_ready.read()==(sc_bit)'0') {
                    huff_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_HUFF;
                }
                break;

            case S_WAIT_HUFF:
                if (huff_ready.read()==(sc_bit)'1') {
                    deq_start = (sc_bit)'1';
                    S_III_Decoder = S_START_DEQ;
                }
                break;

            case S_START_DEQ:
                if (deq_ready.read()==(sc_bit)'0') {
                    deq_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_DEQ;
                }
                break;

            case S_WAIT_DEQ:
                if (deq_ready.read()==(sc_bit)'1')
                    S_III_Decoder = S_CHECK_ST;
                break;

            case S_CHECK_ST:
                if (Mp3HeaderStereo(FrameHeader.read()) && ch==false) {
                    ch = true;
                    scl_start = (sc_bit)'1';
                    S_III_Decoder = S_START_SCALE;
                }
                else {
                    stereo_start = (sc_bit)'1';
                    S_III_Decoder = S_START_STEREO;
                }
                break;

            case S_START_STEREO:
                if (stereo_ready.read()==(sc_bit)'0') {
                    ch = false;
                    fill_stream = (sc_bit)'1';
                    stereo_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_STEREO;
                }
                break;

            case S_WAIT_STEREO:
                if (stereo_ready.read()==(sc_bit)'1') {
                    reorder_start = (sc_bit)'1';
                    S_III_Decoder = S_START_REORDER;
                }
                break;

            case S_START_REORDER:
                if (reorder_ready.read()==(sc_bit)'0') {
                    reorder_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_REORDER;
                }
                break;

            case S_WAIT_REORDER:
                if (reorder_ready.read()==(sc_bit)'1') {
                    antialias_start = (sc_bit)'1';
                    S_III_Decoder = S_START_ANTIALIAS;
                }
                break;

            case S_START_ANTIALIAS:
                if (antialias_ready.read()==(sc_bit)'0') {
                    antialias_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_ANTIALIAS;
                }
                break;

            case S_WAIT_ANTIALIAS:
                if (antialias_ready.read()==(sc_bit)'1') {
                    hybrid_start = (sc_bit)'1';
                    S_III_Decoder = S_START_HYBRID;
                }
                break;

            case S_START_HYBRID:
                if (hybrid_ready.read()==(sc_bit)'0') {
                    hybrid_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_HYBRID;
                }
                break;

            case S_WAIT_HYBRID:
                if (hybrid_ready.read()==(sc_bit)'1') {
                    if (Mp3HeaderStereo(FrameHeader.read()) && ch==false) {
                        ch = true;
                        reorder_start = (sc_bit)'1';
                        S_III_Decoder = S_START_REORDER;
                    }
                    else
                    {
                        subband_start = (sc_bit)'1';
                        S_III_Decoder = S_START_SUBBAND;
                    }
                }
                break;

            case S_START_SUBBAND:
                if (subband_ready.read()==(sc_bit)'0') {
                    subband_start = (sc_bit)'0';
                    S_III_Decoder = S_WAIT_SUBBAND;
                }
                break;

            case S_WAIT_SUBBAND:
                if (subband_ready.read()==(sc_bit)'1')
                    S_III_Decoder = S_CHECK_VERSION;
                break;

            case S_CHECK_VERSION:
                {   ch = false;
                    Mp3FrameHeader frameHeader = FrameHeader.read();
                    if (Mp3HeaderVersion(frameHeader) && gr==false) { // MPEG Ver 1
                        gr = true;

                        scl_start = (sc_bit)'1';
                        S_III_Decoder = S_START_SCALE;
                    }
                    else {    // MPEG Version 2
                        gr = false;
                        hdr_start = (sc_bit)'1';
                        S_III_Decoder = S_WAIT_FILL_STREAM1;
                    }
                }
                break;

            case S_WAIT_FILL_STREAM1:
                if (stream_full.read()==(sc_bit)'1')
                    S_III_Decoder = S_WAIT_FILL_STREAM2;
                break;

            case S_WAIT_FILL_STREAM2:
                fill_stream = (sc_bit)'0';
                S_III_Decoder = S_STOP_PAUSE;
                break;

            case S_STOP_PAUSE:
                if( dIsStopped ){ // Stop Button
                    DoneOut();
                    S_III_Decoder = S_STOP_WAIT;
                    }
                else {
                    gr = false;
                    ch = false;
                    hdr_start = (sc_bit)'1';
                    S_III_Decoder = S_START_HEADER;
                }
                break;

            default:
                S_III_Decoder = S_WAIT_HEADER;
                break;
        }
    }

    channel = ch;
    granule = gr;
}
