///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Decoder_ctl.h
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

#ifndef MP3_DECODER_CTL_INC
#define MP3_DECODER_CTL_INC

#include <systemc.h>
#include "Mp3Common.h"

// Controller FSM
enum S_III_Decoder_type
{
    S_CHECK_BLK,
    S_CHECK_ERR,
    S_CHECK_ST,
    S_CHECK_VERSION,
    S_INIT_MEM,
    S_PAUSE,
    S_READ_MP3,
    S_WAIT_FILL_STREAM,
    S_WAIT_FILL_STREAM1,
    S_WAIT_FILL_STREAM2,
    S_WAIT_FILL_STREAM3,
    S_WAIT_FILL_STREAM4,

    S_WAIT_HEADER,
    S_START_HEADER,
    S_WAIT_SCALE,
    S_START_SCALE,
    S_WAIT_HUFF,
    S_START_HUFF,
    S_WAIT_DEQ,
    S_START_DEQ,
    S_WAIT_STEREO,
    S_START_STEREO,
    S_WAIT_REORDER,
    S_START_REORDER,
    S_WAIT_ANTIALIAS,
    S_START_ANTIALIAS,
    S_WAIT_HYBRID,
    S_START_HYBRID,
    S_WAIT_SUBBAND,
    S_START_SUBBAND,
    S_START_SUBBAND1,

    S_STOP_WAIT,
    S_STOP_PAUSE
};

SC_MODULE(decoder_ctl) {
    sc_slave<>  PlayIn;
    sc_slave<>  StopIn;
    sc_master<> DoneOut;

    sc_in_clk           clk;                // Clock
    sc_in<sc_bit>       rst;                // Reset

    sc_in<sc_bit>       m_init_done;        // Frame FIFO Initializer
    sc_out<sc_bit>      mem_init;

    sc_in<sc_bit>       stream_full;        // Frame FIFO Fill
    sc_out<sc_bit>      fill_stream;

    sc_in<sc_bit>       hdr_ready;          // Header Ready
    sc_out<sc_bit>      hdr_start;          // Header Start

    sc_in<sc_bit>       scl_ready;          // Scale Factor Ready
    sc_out<sc_bit>      scl_start;          // Scale Factor Start

    sc_in<sc_bit>       huff_ready;         // Huffman Ready
    sc_out<sc_bit>      huff_start;         // Huffman Start

    sc_in<sc_bit>       deq_ready;          // Dequantizer Ready
    sc_out<sc_bit>      deq_start;          // Dequantizer Start

    sc_in<sc_bit>       stereo_ready;       // Stereo Ready
    sc_out<sc_bit>      stereo_start;       // Stereo Start

    sc_in<sc_bit>       reorder_ready;      // Reorder Ready
    sc_out<sc_bit>      reorder_start;      // Reorder Start

    sc_in<sc_bit>       antialias_ready;    // Antialias Ready
    sc_out<sc_bit>      antialias_start;    // Antialias Start

    sc_in<sc_bit>       hybrid_ready;       // Hybrid Filter Bank Ready
    sc_out<sc_bit>      hybrid_start;       // Hybrid Filter Bank Start

    sc_in<sc_bit>       subband_ready;      // Subband Synthesis Filter Ready
    sc_out<sc_bit>      subband_start;      // Subband Synthesis Filter Start

    sc_out<sc_bit>      channel;
    sc_out<sc_bit>      granule;

    sc_in<Mp3FrameHeader> FrameHeader;

    // Signals ///////////////////////////////////////////////////////////
    sc_signal<S_III_Decoder_type>   S_III_Decoder;  // FSM State Variable
 
    //-------------------------------------------------------
    // Context declarations
    private:
        int dIsStopped;
        int         nFrame;
        bool        ch; // Channel
        bool        gr; // Granule
 
    //-------------------------------------------------------
    // Thread declarations
    private:
        void proc_decoder_ctl();
        void playInThread(){ dIsStopped = 0; }
        void stopInThread(){ dIsStopped = 1; }

    public:

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(decoder_ctl) : dIsStopped(1) {

        //--------------------------------------
        // Thread bindings

        SC_METHOD(proc_decoder_ctl);
        sensitive << clk << rst;

        SC_SLAVE( playInThread, PlayIn );
        SC_SLAVE( stopInThread, StopIn );

    }
};

#endif
