///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Reorder.h
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

#ifndef III_REORDER_INC
#define III_REORDER_INC

#include <systemc.h>
#include "Mp3Common.h"

enum tReorder_State {
    S_REORDER_WAIT,
    S_REORDER_DECODE,
    S_REORDER_DOWNLOAD,
    S_REORDER_BUSY };

SC_MODULE(iii_reorder)
{
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Reorder Start
    sc_out<sc_bit>      ready;  // Reorder Ready

    sc_in<sc_bit>       channel;
 
    sc_in<Mp3FrameHeader>         FrameHeader;
    sc_in<const Mp3SideHeader *>  SideHeader;

    // Process ///////////////////////////////////////////////////////////
    void proc_reorder();

    // Function proto ////////////////////////////////////////////////////
    void III_reorder (
        int     sampling_frequency,
        int     window_switching_flag,
        int     block_type,
        int     mixed_block_flag,
        int     version,
        int     ch );

    // Globals ///////////////////////////////////////////////////////////
    sc_signal<tReorder_State>   Reorder_State;  // FSM State Variable

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(iii_reorder)
    {
        SC_METHOD(proc_reorder);
        sensitive << clk << rst;
    }
};

#endif
