///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_ScaleFactor.h
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

#ifndef III_SCALEFACTOR_INC
#define III_SCALEFACTOR_INC

#include <systemc.h>
#include "Mp3Common.h"

enum tScale_State {
    S_SCALE_WAIT,
    S_SCALE_DECODE,
    S_SCALE_DOWNLOAD,
    S_SCALE_BUSY };

SC_MODULE(iii_scalefactor) {
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Scale Factor Start
    sc_out<sc_bit>      ready;  // Scale Factor Ready
    
    sc_in<sc_bit>       ch;     // Channel
    sc_in<sc_bit>       gr;     // Granule

    sc_in<Mp3FrameHeader>         FrameHeader;
    sc_in<const Mp3SideHeader *>  SideHeader;
    sc_in<unsigned>               ScfsiBits;

    sc_out<sc_bit>        preflag_out;
    sc_out<unsigned long> part2_start;

    // Process ///////////////////////////////////////////////////////////
    void proc_scale();

    // Signals ///////////////////////////////////////////////////////////
    sc_signal<tScale_State> Scale_State;    // FSM State Variable

    // Member functions //////////////////////////////////////////////////
    void III_get_scale_factors(
                                int             scalefac_compress,
                                int             *scfsi,
                                int             block_type,
                                int             window_switching_flag,
                                int             mixed_block_flag,
                                int             gr,
                                int             ch );

    int III_get_LSF_scale_factors(
                                int             preflag,
                                int             scalefac_comp,
                                int             mode_ext,
                                int             block_type,
                                int             window_switching_flag,
                                int             mixed_block_flag,
                                int             gr,
                                int             ch );
 
    unsigned getBit(){ return GlobalFrameBuffer->GetBits(1); }
    unsigned long getBits(int range)
        {   return GlobalFrameBuffer->GetBits(range); }

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(iii_scalefactor)
    {
        SC_METHOD(proc_scale);
        sensitive << clk << rst << ch << gr;
    }
};

#endif
