///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Stereo.cpp
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

#include "math.h"
#include "sc_III_Stereo.h"

#include "sc_III_Decoder_Mem.h"
#include "sfBandIndex.h"

void iii_stereo::proc_stereo()
{

    if (rst.read()==(sc_bit)'0')    // Async. Reset
    {
        Stereo_State = S_STEREO_WAIT;
        ready = (sc_bit)'1';
    }
    else if (clk.event() && clk.read()==(sc_bit)'1')    // Clocking Rising-Edge
    {
        switch(Stereo_State.read())
        {
            case S_STEREO_WAIT:
                if (start.read()==(sc_bit)'1')
                {
                    ready = (sc_bit)'0';
                    Stereo_State = S_STEREO_DECODE;
                }
                break;

            case S_STEREO_DECODE:
                if (start.read()==(sc_bit)'0') {
                    Mp3FrameHeader frameHeader = FrameHeader.read();
                    const Mp3SideHeader *sideHeader = SideHeader.read();

                    III_stereo(
                        Mp3HeaderStereo(frameHeader),
                        Mp3HeaderMode(frameHeader),
                        Mp3HeaderModeExt(frameHeader),
                        sideHeader->ScalefacCompress,
                        Mp3HeaderSamplingFrequency(frameHeader),
                        Mp3HeaderVersion(frameHeader),
                        sideHeader->WindowSwitchingFlag,
                        sideHeader->BlockType,
                        sideHeader->MixedBlockFlag );

                    Stereo_State = S_STEREO_DOWNLOAD;
                }
                break;

            case S_STEREO_DOWNLOAD:
                Stereo_State = S_STEREO_BUSY;
                break;

            case S_STEREO_BUSY:
                ready = (sc_bit)'1';
                Stereo_State = S_STEREO_WAIT;
                break;

            default:
                Stereo_State = S_STEREO_WAIT;
                break;
        }
    }
}

///////////////////////////////////////////////////////////////////////////
// Stereo Processing of Layer III
//
void iii_stereo::III_stereo(
    int     stereo,
    int     mode,
    int     mode_ext,
    int     scalefac_compress,
    int     sampling_freq,
    int     version,
    int     window_switching_flag,
    int     block_type,
    int     mixed_block_flag )
{
    int sfreq       = sampling_freq + (version * 3);
    int ms_stereo   = (mode==MPG_MD_JOINT_STEREO) && (mode_ext & 0x2); 
    int i_stereo    = (mode==MPG_MD_JOINT_STEREO) && (mode_ext & 0x1);
    
    int     sfb;
    int     i,j,sb,ss;
    short   is_pos[SBLIMIT*SSLIMIT]; 
    double  is_ratio[SBLIMIT*SSLIMIT];
    double  io;
    double  k[2][SBLIMIT*SSLIMIT];

    if( (scalefac_compress % 2) == 1 )
        io = (double)0.707106781188;
    else
        io = (double)0.840896415256;

    // intialization
    for ( i=0; i<SBLIMIT*SSLIMIT; i++ )
        is_pos[i] = 7;

//  if ((stereo == 2) && i_stereo )
    if (stereo && i_stereo )
    {
        if (window_switching_flag && (block_type == 2))
        {
            if(mixed_block_flag)
            {
                int max_sfb = 0;

                for ( j=0; j<3; j++ )
                {
                    int sfbcnt;

                    sfbcnt = 2;
                    for( sfb=12; sfb >=3; sfb-- )
                    {
                        int lines;
                        
                        lines = sfBandIndex[sfreq].s[sfb+1]-sfBandIndex[sfreq].s[sfb];
                        i     = 3*sfBandIndex[sfreq].s[sfb] + (j+1) * lines - 1;

                        while ( lines > 0 )
                        {
                            if ( GlobalDecoderMem->get_ro(1, i/SSLIMIT, i%SSLIMIT) != 0.0 )
                            {
                                sfbcnt = sfb;
                                sfb    = -10;
                                lines  = -10;
                            }
                            lines--;
                            i--;
                        }
                    }
                    sfb = sfbcnt + 1;

                    if ( sfb > max_sfb )
                        max_sfb = sfb;

                    while( sfb<12 )
                    {
                        sb = sfBandIndex[sfreq].s[sfb+1]-sfBandIndex[sfreq].s[sfb];
                        i  = 3*sfBandIndex[sfreq].s[sfb] + j * sb;

                        for ( ; sb > 0; sb--)
                        {
                            is_pos[i] = GlobalDecoderMem->get_scalefac( 1, j, sfb, 1);
                            //is_pos[i] = u_scale->(*scalefac)[1].s[j][sfb];
                            if ( is_pos[i] != 7 )
                                if( version==0 )
                                    III_i_stereo_k_values(is_pos[i],io,i,k);
                                else
                                    is_ratio[i] = tan((double)is_pos[i] * (PI / 12));
                            i++;
                        }
                        sfb++;
                    }

                    sb  = sfBandIndex[sfreq].s[12]-sfBandIndex[sfreq].s[11];
                    sfb = 3*sfBandIndex[sfreq].s[11] + j * sb;
                    sb  = sfBandIndex[sfreq].s[13]-sfBandIndex[sfreq].s[12];

                    i = 3*sfBandIndex[sfreq].s[11] + j * sb;

                    for ( ; sb > 0; sb-- )
                    {
                        is_pos[i]   = is_pos[sfb];
                        is_ratio[i] = is_ratio[sfb];
                        k[0][i] = k[0][sfb];
                        k[1][i] = k[1][sfb];
                        i++;
                    }
                }

                if ( max_sfb <= 3 )
                {
                    i = 2;
                    ss = 17;
                    sb = -1;

                    while ( i >= 0 )
                    {
                        if ( GlobalDecoderMem->get_ro(1, i, ss) != 0.0 )
                        {
                            sb = i*18+ss;
                            i = -1;
                        }
                        else
                        {
                            ss--;

                            if ( ss < 0 )
                            {
                                i--;
                                ss = 17;
                            }
                        }
                    }
                    i = 0;
                    while ( sfBandIndex[sfreq].l[i] <= sb )
                        i++;

                    sfb = i;
                    i = sfBandIndex[sfreq].l[i];
                    for ( ; sfb<8; sfb++ )
                    {
                        sb = sfBandIndex[sfreq].l[sfb+1]-sfBandIndex[sfreq].l[sfb];
                        for ( ; sb > 0; sb--)
                        {
                            is_pos[i] = GlobalDecoderMem->get_scalefac( 1, 0, sfb, 0);
                            //is_pos[i] = u_scale->(*scalefac)[1].l[sfb];
                            if ( is_pos[i] != 7 )
                                if ( version==0 )
                                    III_i_stereo_k_values(is_pos[i],io,i,k);
                                else
                                    is_ratio[i] = tan((double)is_pos[i] * (PI / 12));
                                i++;
                        }
                    }
                }
            }
            else
            {
                for ( j=0; j<3; j++ )
                {
                    int sfbcnt;
                    sfbcnt = -1;
                    for( sfb=12; sfb >=0; sfb-- )
                    {
                        int lines;
                        lines = sfBandIndex[sfreq].s[sfb+1]-sfBandIndex[sfreq].s[sfb];
                        i = 3*sfBandIndex[sfreq].s[sfb] + (j+1) * lines - 1;
                    
                        while ( lines > 0 )
                        {
                            if ( GlobalDecoderMem->get_ro( 1, i/SSLIMIT, i%SSLIMIT) != 0.0 )
                            {
                                sfbcnt = sfb;
                                sfb = -10;
                                lines = -10;
                            }
                            lines--;
                            i--;
                        }
                    }
                    sfb = sfbcnt + 1;
                    
                    while( sfb<12 )
                    {
                        sb = sfBandIndex[sfreq].s[sfb+1]-sfBandIndex[sfreq].s[sfb];
                        i = 3*sfBandIndex[sfreq].s[sfb] + j * sb;
                        for ( ; sb > 0; sb--)
                        {
                            is_pos[i] = GlobalDecoderMem->get_scalefac( 1, j, sfb, 1);
                            if ( is_pos[i] != 7 )
                                if( version==0 )
                                    III_i_stereo_k_values(is_pos[i],io,i,k);
                                else
                                    is_ratio[i] = tan( (double)is_pos[i] * (PI / 12));
                                i++;
                        }
                        sfb++;
                    }

                    sb = sfBandIndex[sfreq].s[12]-sfBandIndex[sfreq].s[11];
                    sfb = 3*sfBandIndex[sfreq].s[11] + j * sb;
                    sb = sfBandIndex[sfreq].s[13]-sfBandIndex[sfreq].s[12];

                    i = 3*sfBandIndex[sfreq].s[11] + j * sb;
                    for ( ; sb > 0; sb-- )
                    {
                        is_pos[i] = is_pos[sfb];
                        is_ratio[i] = is_ratio[sfb];
                        k[0][i] = k[0][sfb];
                        k[1][i] = k[1][sfb];
                        i++;
                    }
                }
            }
        }
        else
        {
            i = 31;
            ss = 17;
            sb = 0;
            
            while ( i >= 0 )
            {
                if ( GlobalDecoderMem->get_ro(1, i, ss) != 0.0 )
                {
                    sb = i*18+ss;
                    i = -1;
                }
                else
                {
                    ss--;
                    if ( ss < 0 )
                    {
                        i--;
                        ss = 17;
                    }
                }
            }
            i = 0;

            while ( sfBandIndex[sfreq].l[i] <= sb )
                i++;
            sfb = i;
            i = sfBandIndex[sfreq].l[i];
            for ( ; sfb<21; sfb++ )
            {
                sb = sfBandIndex[sfreq].l[sfb+1] - sfBandIndex[sfreq].l[sfb];
                for ( ; sb > 0; sb--)
                {
                    is_pos[i] = GlobalDecoderMem->get_scalefac( 1, 0, sfb, 0);
                    if ( is_pos[i] != 7 )
                        if( version==0 )
                            III_i_stereo_k_values(is_pos[i],io,i,k);
                        else
                            is_ratio[i] = tan((double)is_pos[i] * (PI / 12));
                        i++;
                }
            }
            sfb = sfBandIndex[sfreq].l[20];
            for ( sb = 576 - sfBandIndex[sfreq].l[21]; sb > 0; sb-- )
            {
                is_pos[i] = is_pos[sfb];
                is_ratio[i] = is_ratio[sfb];
                k[0][i] = k[0][sfb];
                k[1][i] = k[1][sfb];
                i++;
            }
        }
    }

    // Initialize
    if (stereo) { // Stereo
        for(sb=0;sb<SBLIMIT;sb++) {
            for(ss=0;ss<SSLIMIT;ss++) {
                i = (sb*18)+ss;
                if ( is_pos[i] == 7 ) {
                    if ( ms_stereo ) {
                        double  xr;

                        xr = (GlobalDecoderMem->get_ro(0, sb, ss)
                                + GlobalDecoderMem->get_ro(1, sb, ss))/(double)1.41421356;
                        GlobalDecoderMem->put_lr( xr, 0, sb, ss);

                        xr = (GlobalDecoderMem->get_ro(0, sb, ss)
                                - GlobalDecoderMem->get_ro(1, sb, ss))/(double)1.41421356;
                        GlobalDecoderMem->put_lr( xr, 1, sb, ss);
                    }
                    else {
                        GlobalDecoderMem->put_lr(
                            GlobalDecoderMem->get_ro(0, sb, ss), 0, sb, ss);
                        GlobalDecoderMem->put_lr(
                            GlobalDecoderMem->get_ro(1, sb, ss), 1, sb, ss);
                    }
                }
                else if (i_stereo ) {
                    if ( version==0 ) {
                        double  xr;

                        xr = GlobalDecoderMem->get_ro(0, sb, ss) * k[0][i];
                        GlobalDecoderMem->put_lr( xr, 0, sb, ss);

                        xr = GlobalDecoderMem->get_ro(0, sb, ss) * k[1][i];
                        GlobalDecoderMem->put_lr( xr, 1, sb, ss);

                        //lr[0][sb][ss] = xr[0][sb][ss] * k[0][i];
                        //lr[1][sb][ss] = xr[0][sb][ss] * k[1][i]; 
                    }
                    else {
                        double  xr;

                        xr = GlobalDecoderMem->get_ro(0, sb, ss) * (is_ratio[i]/(1+is_ratio[i]));
                        GlobalDecoderMem->put_lr( xr, 0, sb, ss);

                        xr = GlobalDecoderMem->get_ro(0, sb, ss) * (1/(1+is_ratio[i]));
                        GlobalDecoderMem->put_lr( xr, 1, sb, ss);

                        //lr[0][sb][ss] = xr[0][sb][ss] * (is_ratio[i]/(1+is_ratio[i]));
                        //lr[1][sb][ss] = xr[0][sb][ss] * (1/(1+is_ratio[i])); 
                    }
                }
                else
                    fprintf(stderr, "Error in streo processing\n");
            }
        }
    }
    else {  // Mono
        for(sb=0;sb<SBLIMIT;sb++)
            for(ss=0;ss<SSLIMIT;ss++)
                GlobalDecoderMem->put_lr(
                    GlobalDecoderMem->get_ro(0, sb, ss), 0, sb, ss);
                //lr[0][sb][ss] = xr[0][sb][ss];
    }
}


///////////////////////////////////////////////////////////////
// K-value for I-Stereo
void iii_stereo::III_i_stereo_k_values(
    int is_pos,
    double io,
    int i,
    double k[2][576] )
{
    if(is_pos == 0)
    { 
        k[0][i] = 1;
        k[1][i] = 1;
    }
    else if ((is_pos % 2) == 1)
    {
        k[0][i] = pow(io,(double)((is_pos + 1)/2));
        k[1][i] = 1;
    }
    else
    {
        k[0][i] = 1;
        k[1][i] = pow(io,(double)(is_pos/2));
    }
}
