///////////////////////////////////////////////////////////////////////////
// Filename : sc_III_Stereo.h
// Project  : Ans_MP3_Player/MP3_Decoder
// Author   : GoodKook / goodkook@nms.anslab.co.kr
// Company  : AnsLab Co. / http://www.anslab.co.kr
// Revision : 5-4-01: Modified for interface to untimed SystemC testbench
//                    - John Stickley, Ikos Systems, Inc. (johns@ikos.com)
// Comment  :
//
//

#ifndef III_STEREO_INC
#define III_STEREO_INC

#include <systemc.h>
#include "Mp3Common.h"

enum tStereo_State {
    S_STEREO_WAIT,
    S_STEREO_DECODE,
    S_STEREO_DOWNLOAD,
    S_STEREO_BUSY };

SC_MODULE(iii_stereo)
{
    sc_in_clk           clk;    // Clock
    sc_in<sc_bit>       rst;    // Reset

    sc_in<sc_bit>       start;  // Stereo Start
    sc_out<sc_bit>      ready;  // Stereo Ready
 
    sc_in<Mp3FrameHeader>         FrameHeader;
    sc_in<const Mp3SideHeader *>  SideHeader;

    // Process ///////////////////////////////////////////////////////////
    void proc_stereo();

    // FSM ///////////////////////////////////////////////////////////////
    sc_signal<tStereo_State>    Stereo_State;   // FSM State Variable

    // Prototypes ////////////////////////////////////////////////////////
    void III_stereo(
        int     stereo,
        int     mode,
        int     mode_ext,
        int     scalefac_compress,
        int     sampling_freq,
        int     version,
        int     window_switching_flag,
        int     block_type,
        int     mixed_block_flag );

    void III_i_stereo_k_values(
        int is_pos,
        double io,
        int i,
        double k[2][576] );

    // Constructor ///////////////////////////////////////////////////////////
    SC_CTOR(iii_stereo)
    {
        SC_METHOD(proc_stereo);
        sensitive << clk << rst;
    }
};

#endif
